from slack_sdk import WebClient

import logging


USER_ICON = ':fire:'
UPDATE_ICON = ':recycle:'
SUCCESS_ICON = ':white_check_mark:'
ERROR_ICON = ':x:'
SKIP_ICON = ':fast_forward:'
ENV_ICON = ':hocho:'


HEADER_MSGS = {
    'start': f'{UPDATE_ICON} Executando...',
    'success': f'{SUCCESS_ICON} Sucesso',
    'error': f'{ERROR_ICON} Erro'
}

class Notify():
    def __init__(self, slack_token, slack_channel, step, tenant="", project="", enable_slack_notifications=True):
        """Initializes the slack notifier

        Args:
            slack_token (str): Slack API token.
            slack_channel (str): Name of the slack channel to be used.
            step (str): Step name.
            tenant (str): Tenant name.
            project (str): Project name.
            enable_slack_notifications (bool, optional): Whether to enable notifications \
                or not. Defaults to True.
        """
        self.SLACK_TOKEN = slack_token
        self.ENABLE_SLACK_NOTIFICATIONS = enable_slack_notifications
        self.SLACK_CHANNEL = slack_channel
        self.STEP = step
        self.PROJECT = project
        self.TENANT = tenant
        self.SLACK_THREAD_TS = None
        self.SLACK_CHANNEL_ID = None
        
    def is_slack_enabled(self):
        return (
            self.SLACK_TOKEN and self.SLACK_TOKEN!='' and self.ENABLE_SLACK_NOTIFICATIONS
        )
        
    def get_title(self, step, tenant, project, msg_type):
        tenant_string = f'[{tenant}]' if tenant else ""
        project_string = f'[{project}]' if project else ""
        return [
            {
                "type": "header",
                "text": {
                    "type": "plain_text",
                    "text": f'{tenant_string} {project_string} {step} - {HEADER_MSGS[msg_type]}',
                    "emoji": True
                }
            }
        ]

    def start(self):
        logging.info(f"[Notify:start] Start of [{self.STEP}]")
        
        if self.is_slack_enabled():
            
            client = WebClient(token=self.SLACK_TOKEN)
            response = client.chat_postMessage(
                channel= self.SLACK_CHANNEL,
                blocks= self.get_title(self.STEP,self.TENANT,self.PROJECT, 'start'),
                icon_emoji= USER_ICON
            )
            
            self.SLACK_THREAD_TS = response.get("ts")
            self.SLACK_CHANNEL_ID = response.get("channel")



    def add_env_config(self, logging_config):
        self.add_update(logging_config, ENV_ICON)


    def add_update(self,message, icon=UPDATE_ICON):
        logging.info(f"[Notify:update]: {message}")

        if self.is_slack_enabled() and self.SLACK_THREAD_TS:
            client = WebClient(token=self.SLACK_TOKEN)
            client.chat_postMessage(
                channel= self.SLACK_CHANNEL,
                text= message,
                icon_emoji= icon,
                thread_ts= self.SLACK_THREAD_TS
            )


    def finish(self,success=False):
        logging.info(f"[Notify:finish] Finish of [{self.STEP}]")

        if self.is_slack_enabled():
            msg_type = 'success' if success else 'error'
            client = WebClient(token=self.SLACK_TOKEN)
            client.chat_update(
                channel= self.SLACK_CHANNEL_ID,
                ts= self.SLACK_THREAD_TS,
                blocks= self.get_title(self.STEP, self.TENANT, self.PROJECT, msg_type),
                icon_emoji= USER_ICON
            )




#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup
from pkgver import package_version
with open('README.rst') as readme_file:
    readme = readme_file.read()

requirements = [
    'slack-sdk>=3.19.1',
    'mixpanel>=4.10.0'
]

setup(
    name='scora-notify',
    version=package_version,
    description="Scora Slack notification lib",
    long_description=readme,
    author="Oncase",
    author_email='contato@oncase.com.br',
    package_dir = {"scora_notify": "src"},
    packages=['scora_notify','pkgver'],
    zip_safe=True,
    install_requires=requirements,
    extras_require={
        'dev': [
            'twine>=3.1.1',
            'Sphinx==3.1.1',
            'sphinxcontrib-napoleon==0.7',
            'sphinx-rtd-theme==0.5.0',
            'sphinx-click==2.3.2',
            'flake8==3.8.3'
        ]
    },
    keywords='scora',
    classifiers=[
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.6',
    ]
)



from mixpanel import Mixpanel
import uuid
from datetime import datetime
import re

import logging

class EventTracker():
    
    def __init__(self, mixpanel_token, project, tenant, is_enabled=True):
        """Initializes the event tracker

        Args:
            mixpanel_token (str): Mixpanel Project token. (refering to the Performance Analytics Project)
            project (str): Name of the project being tracked. (refering to the code's project)
            tenant (str): Tenant name. (refering to the Performance Analytics Project)
            is_enabled (bool, optional): Whether to enable tracking. Defaults to True.
        """
        self.MIXPANEL_TOKEN = mixpanel_token
        self.PROJECT = project
        self.TENANT = tenant
        self.IS_ENABLED = is_enabled
        
        if not self.IS_ENABLED: return
        
        self.mp = Mixpanel(self.MIXPANEL_TOKEN)
        self.session_id = uuid.uuid1().int
        
        self.event_time_start = {}
        
    def _validate_event_name(self,event_name):
        if type(event_name)!=str:
            raise ValueError("Make sure event name is a string")
           
    def _get_parsed_event_name(self,event_name):
        self._validate_event_name(event_name)
        
        event_name= re.sub(r' |-', '_', event_name)
        
        event_name = f"{self.PROJECT}_{event_name}"
        return event_name.upper()
    
    def _add_context_to_properties(self, properties):
        return {**properties, "Project":self.PROJECT, "Tenant": self.TENANT}
    
    def track(self, event_name, properties={}):
        if not self.IS_ENABLED: return
        
        event_name = self._get_parsed_event_name(event_name)
        properties = self._add_context_to_properties(properties)
        
        if event_name in self.event_time_start:
            delta = datetime.now() - self.event_time_start[event_name] 
            duration = delta.total_seconds()
            properties = {**properties, 'Duration':duration}
            del self.event_time_start[event_name]

        self.mp.track(self.session_id, event_name, properties)
        
        logging.info(f"Event {event_name} successfully tracked")
       
    def time_event(self, event_name):
        if not self.IS_ENABLED: return
        
        event_name = self._get_parsed_event_name(event_name)
        
        self.event_time_start[event_name] = datetime.now()